/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * @param annozilla Annozilla object
 * @param annotation if updating, an annotation.
 *                   if new post, then null or a string indicating the type.
 * @param xptr if updating, then null.  if new post, then xptr string.
 */
function PostObserver(
    annotation,
    baseURL,
    xptr,
    bodyURI,
    updateFunction,
    defaultCreator,
    defaultLanguage,
    defaultCharset
){
    this.update = updateFunction;

    this.bundle = srGetStrBundle(
        "chrome://annozilla/locale/annozilla.properties"
    );
    this.editorBundle = srGetStrBundle(
        "chrome://annozilla/locale/editor.properties"
    );

    if (typeof annotation == 'string') {
        this.type = annotation; //annotation param may be type or null
        this.annotation = null; //and not a pre-existing annotation
    } else {
        this.annotation = annotation;
        this.type = null;
    }
    this.baseURL = baseURL;
    this.bodyURI = bodyURI;
    this.xptr = xptr;

    var service = Components.classes[
        PostObserver.ANNOTATIONS_SERVICE_CONTRACTID
    ].getService().QueryInterface(Components.interfaces.nsIAnnotationsService);
    this.app = service.registerApplication("annozilla");

    this.server = this.findDefaultServer();
    
    this.language = null;

    this.charset = defaultCharset;
    
    // if we're updating, grab the old information
    if (this.annotation) {
        // type
        this.type=this.annotation.data.getProperty(PostObserver.RDF_NS, "type");
        if (this.type) {
            // slice the namespace off the type
            var shortType = PostObserver.extractProperty(this.type);
            if (shortType) {
	      this.type = shortType;
            }
        }
      
        // language
        this.language = this.annotation.data.getProperty(
            PostObserver.DC10_NS,
            "language"
        );

        // creator
        this.creator = annotation.data.getProperty(
            PostObserver.DC10_NS,
            "creator"
        );
    }
    // otherwise, if posting
    else {
        this.creator = defaultCreator;
    }
    
    // if we're not updating, or if the annotation didn't have the
    // necessary properties, set them to defaults
    if (!this.type) {
        this.type = PostObserver.DEFAULT_ANNOTATION_TYPE;
    }
    if (!this.language) {
        // try to get the langugage pref
        this.language = defaultLanguage;
        if (! this.language) {
            this.language = this.bundle.GetStringFromName("DEFAULT_LANGUAGE");
        }
    }
    
    if ( (!this.creator) || (this.creator == "") ) {
        this.creator = "Anonymous";
    }

    // add the postobserver as an annotations observer
    this.app.addObserver(this);
}

PostObserver.DEFAULT_ANNOTATION_TYPE = "Comment";

PostObserver.PROFILE_DIRECTORY = "ProfD";
PostObserver.SERVERS_FILENAME = "annozillaServers.rdf";
PostObserver.SERVERS_ROOT_URN = "urn:annotations:servers";

PostObserver.ANNOTATIONS_SERVICE_CONTRACTID = "@mozilla.org/annotations_service;1";
PostObserver.XMLSERIALIZER_CONTRACTID="@mozilla.org/xmlextras/xmlserializer;1";
PostObserver.METADATA_CONTRACTID = "@mozilla.org/annotation-metadata;1";

PostObserver.ANNOT_NS = "http://www.w3.org/2000/10/annotation-ns#";
PostObserver.RDF_NS   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
PostObserver.DC10_NS  = "http://purl.org/dc/elements/1.0/";
PostObserver.DC11_NS  = "http://purl.org/dc/elements/1.1/";
PostObserver.THREAD_NS= "http://www.w3.org/2001/03/thread#";
PostObserver.XHTML_NS = "http://www.w3.org/1999/xhtml";
PostObserver.TYPE_NS  = "http://www.w3.org/2000/10/annotationType#";

// Values for the state.
PostObserver.UNCHANGED = 0;
PostObserver.DELETED = 1;

PostObserver.prototype = {
    annozilla: undefined,
    annotation: undefined,
    xptr: undefined,

    // all nsIAnnotationObserver functions defined inline here
    autoLoadOnBeginPageLoad: function(url, win) {},
    autoLoadOnSuccessPageLoad: function(url, win) {},
    autoLoadOnFailPageLoad: function(url, win) {},
    autoLoadAnnotationsForServer: function(
        serverURI,
        win,
        annotatedURI,
        annotationsLength,
        annotations
    ){},
    autoLoadFailureForServer: function(serverURI, win, annotatedURI, message){},
    deleteAnnotationOnSuccess: function(url) {
        // if we have deleted the annotation we were trying to update
        this.success = false;
        this.state = PostObserver.DELETED;
        this.update();
    },
    deleteAnnotationOnFailure: function(url, message) {},
    getRepliesOnSuccess: function(annotation, repliesLength, replies) {},
    getRepliesOnFailure: function(annotation, message) {},
    replyToAnnotationOnSuccess: function(original, reply) {},
    replyToAnnotationOnFailure: function(original, message) {},
    loadAnnotationsOnSuccess: function(
        serverURI,
        annotatedURI,
        annotationsLength,
        annotations
    ){},
    loadAnnotationsOnFailure: function(serverURI, annotatedURI, message) {},
    loadAnnotationBodyOnSuccess: function(annotation, bodyText) {},
    loadAnnotationBodyOnFailure: function(annotation, message) {},

    postAnnotationOnSuccess: function(newAnnotation) {
        this.success = true;
        this.state = PostObserver.UNCHANGED;
        this.annotation = newAnnotation;
        this.update();
    },

    postAnnotationOnFailure: function(message) {
        this.success = false;
        this.failureMessage = message;
        this.update();
    },

    updateAnnotationOnSuccess: function(updatedAnnotation) { 
        this.success = true;
        this.state = PostObserver.UNCHANGED;
        this.annotation = updatedAnnotation;
        this.update();
    },

    updateAnnotationOnFailure: function(url, message) {
        this.success = false;
        this.failureMessage = message;
        this.update();
    }
};

/**
 * Splits a namespace + property string and 
 * returns the property.
 *
 * @param resourceString string of namespace + property
 * @return object with namespace and property fields
 */
PostObserver.extractProperty = function(resourceString) {
    var i;
    for (i = (resourceString.length - 1); i >= 0; i--) {
	var charAtI = resourceString.charAt(i);
	if (charAtI == '#' || charAtI == '/' || charAtI == ':') {
	    return resourceString.substring(i+1);
	}
    }
    
    // couldn't find it -- return null
    return null;
}

PostObserver.prototype.stopObserving = function(){
    this.app.removeObserver(this);
}

PostObserver.prototype.findDefaultServer = function() {
    return this.app.getDefaultPostServer();
}

PostObserver.prototype.deleteAnnotation = function() {
    this.app.deleteAnnotation(this.annotation);
}

// Returns false on failure and true on success.
PostObserver.prototype.post = function(xml) {

    // first off, ensure that this server is still around
    if (! this.app.hasServer(this.server)) {
      if (this.annotation) return false;
      else return false;
    }

    var date = PostObserver.formatDate();

    // updating
    if(this.annotation){
        this.annotation.setBodyText(xml);
        this.annotation.data.setLiteralProperty(
            PostObserver.DC10_NS,
            "language",
            this.language
        );
        this.annotation.data.setResourceProperty(
            PostObserver.RDF_NS,
            "type",
            PostObserver.TYPE_NS + this.type
        );
        this.annotation.data.setLiteralProperty(
            PostObserver.DC10_NS,
            "date",
            date
        );
        this.app.updateAnnotation(this.annotation);
    }
    // posting
    else {
    
        var data = Components.classes[
            PostObserver.METADATA_CONTRACTID
        ].createInstance(Components.interfaces.nsIAnnotationMetadata);

        data.setRequiredProperties(
            PostObserver.TYPE_NS + this.type,
            this.baseURL,
            this.baseURL + "#" + this.xptr,
            this.creator,
            date,
            date
        );
        data.setLiteralProperty(PostObserver.DC10_NS,"language",this.language);
        data.setResourceProperty(
            PostObserver.RDF_NS,
            "type",
            PostObserver.TYPE_NS + this.type
        );

        this.app.postAnnotation(data, xml, this.charset, this.server);
    }

    return true;
}

PostObserver.formatDate = function() {
  var operandStr;

    // current time
    var now  = new Date(); 

    // regexp to add leading 0
    var reg = new RegExp(/^([0-9])$/); 

    // standard part of string common to all time zones
    var formatted = now.getUTCFullYear() + "-" +
    new String(now.getMonth() + 1).replace(reg, "0$1") + "-" +
    new String(now.getDate()).replace(reg, "0$1") + "T" +
    new String (now.getHours()).replace(reg, "0$1") + ":" +
    new String(now.getMinutes()).replace(reg, "0$1") + ":" +
    new String(now.getSeconds()).replace(reg, "0$1");

    // get timezone offset operandStr
    var offset = now.getTimezoneOffset();
    if (offset < 0) {
	operandStr = "+";
    }
    else if (offset > 0) {
	operandStr = "-";
    }
    // special case for +0 GMT
    else {
	return formatted + "Z";
    }

    // if offset less than zero, make it positive
    if (offset < 0) {
	offset *= -1;
    }

    // calculate hours and minutes
    var hours = Math.floor(offset / 60);
    var minutes = offset % 60;
    
    return formatted + operandStr + 
    new String(hours).replace(reg, "0$1") + ":" +
    new String(minutes).replace(reg, "0$1");
}
