/* -*- Mode: Java; tab-width: 2; c-basic-offset: 2; -*-
 * 
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation.  All Rights
 * Reserved.
 *
 * Contributor(s):
 *  Doug Daniels <rainking@rice.edu> -- morphed code for Helper Applications pref window
 *                                      into multiple servers pref window for Annozilla
 *                   
 */

// Constants
const NS_LOCAL_FILE_CONTRACTID = "@mozilla.org/file/local;1";
const NS_LOCALFILEOUTPUTSTREAM_CONTRACTID =
                              "@mozilla.org/network/file-output-stream;1";
const NS_RDFSERVICE_CONTRACTID = "@mozilla.org/rdf/rdf-service;1";

// File read/write operations
const NS_WRONLY      = 0x02;
const NS_CREATE_FILE = 0x08;
const NS_TRUNCATE    = 0x20;

// user -- read, write; group -- read; other -- read
const DEFAULT_FILE_PERMS = 00400 | 00200 | 00040 | 00004;

var gList   = null;
var gDS     = null;
var gSeq    = null;

var gNewServerRV = null;

var gNewServerButton  = null;
var gEditServerButton     = null;
var gRemoveServerButton   = null;

var gLocalServerResource = gRDF.GetResource(LOCALSERVER_URL);

/**
 * Initializes this preferences window.  
 * Called by XUL upon loading.
 */
function Startup()
{
    // set up the elements
    gList = document.getElementById("serverList");
    gNewServerButton  = document.getElementById("newServerButton");
    gEditServerButton     = document.getElementById("editServerButton");
    gRemoveServerButton   = document.getElementById("removeServerButton");

    // get the preferences directory
    var file = getServersFile();

    // if the file does not exist, create it
    if (!file.exists()) {
        createNewAnnozillaServersFile(file);
    }

    // get the data source
    gDS = getDatasource (file);

    // intialize the listbox
    gList.database.AddDataSource(gDS);
    gList.setAttribute("ref", SERVERS_ROOT_URN);

    // build a seq container
    gSeq = RDFU.makeSeq(gDS, gRDF.GetResource(SERVERS_ROOT_URN));

    // setup localServer
    maintainLocalServer();

    // TODO check whether the only server is the default post server?
    if (gSeq.GetCount() <= 1) {
        // Use basic prefs
        document.getElementById ("basicPrefsUsed").checked = true;
        document.getElementById('serversDeck').selectedIndex = 0;

        if (gSeq.GetCount() == 1) {
            try {
                var elements = gSeq.GetElements();
                var server = elements.getNext();
                var serverURL = server.QueryInterface(Components.interfaces.nsIRDFResource).Value;
                if (serverURL == LOCALSERVER_URL) {
                    document.getElementById ("basicServerType").value = "local";
                } else {
                    document.getElementById ("basicServerType").value = "remote";
                    document.getElementById ("serverURL").value = serverURL;
                    document.getElementById ("user").value =
                        getLiteralFromResource(gDS, server, "user");
                    document.getElementById ("password").value =
                        getLiteralFromResource(gDS, server, "password");
                    document.getElementById ("confirmation").value =
                        getLiteralFromResource(gDS, server, "password");
                }
            } catch (ex) { window.alert (ex); }
        }
    } else {
        document.getElementById ("basicPrefsUsed").checked = false;
        document.getElementById('serversDeck').selectedIndex = 1;
        document.getElementById('switchToBasic').disabled = true;
    }
}

function maintainLocalServer() {
    var inList = isLocalServerInList();
    document.getElementById("addLocalServerButton").hidden = inList;
    document.getElementById("removeLocalServerButton").hidden = !inList;
}

// Called on initialisation and after add/remove server to keep up the
// state of the 'Basic Options'/'Advanced Options'
function maintainModeButtons () {
    if (gSeq.GetCount() > 1) {
        document.getElementById('switchToBasic').disabled = true;
    } else {
        document.getElementById('switchToBasic').disabled = false;
    }
}

function isLocalServerInList() {
    return gSeq.IndexOf(gLocalServerResource) != -1;
}

function addLocalServer() {

    gSeq.AppendElement(gLocalServerResource);

    // assert the literals for the localServer
    assertForResource(gLocalServerResource, "user", "noUser");
    assertForResource(gLocalServerResource, "password", "noPass");
    
    // flush the ds to disk.   
    var remoteDS = gDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    remoteDS.Flush();

    maintainLocalServer();
}

function removeLocalServer() {
    deleteServerFromRDF(LOCALSERVER_URL);
    
    // rebuild the list
    gList.builder.rebuild();
    gList.setAttribute("ref", SERVERS_ROOT_URN);
    
    // setup localServer
    maintainLocalServer();
}

/**
 * Assert a literal for a resource in the global
 * datasource.
 *
 * @param aResource Resource to assert the literal for
 * @param aPropertyString literal to assert
 * @param aValueString value for that literal
 */
function assertForResource(aResource, aPropertyString, aValueString)
{
  var valueProperty = gRDF.GetResource(SERVER_RDF(aPropertyString));
  var serverLiteral = gRDF.GetLiteral(aValueString);
  gDS.Assert(aResource, valueProperty, serverLiteral, true);
}

// TODO duplicated from prefs.js
function getLiteralFromResource(aDataSource, aResource, aPropertyString) {
    try {
        var valueProperty = gRDF.GetResource(SERVER_RDF(aPropertyString));
        var node = aDataSource.GetTarget(aResource, valueProperty, true);
        return node.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
    } catch(e) {
        return null;
    }
}


/**
 * Create a new server entry.  Blocks upon
 * opening dialog window.
 */
function newServer()
{
    window.openDialog
        ("chrome://annozilla/content/prefs/pref-annozilla-servers-new.xul",
         "newServer",
         "chrome,modal=yes,resizable=no");

    if (gNewServerRV) {
        gList.builder.rebuild();
        gList.setAttribute("ref", SERVERS_ROOT_URN);
        gNewServerRV = null;

        maintainLocalServer();
        maintainModeButtons();
    } else {
        dump("No gNewServerRV!\n");
    }
}

/**
 * Remove the currently selected server.
 */
function removeServer()
{
    if (gList.selectedItems && gList.selectedItems[0]) {
        var uri = gList.selectedItems[0].id;
        deleteServerFromRDF(uri);
        gList.setAttribute("ref", SERVERS_ROOT_URN);

        // setup localServer
        maintainLocalServer();
        maintainModeButtons();
    }
}

/**
 * Edit the currently selected server.  Blocks upon opening
 * dialog window.
 */
function editServer()
{
    if (gList.selectedItems && gList.selectedItems[0]) {
        var uri = gList.selectedItems[0].id;
        if (uri == LOCALSERVER_URL) {
            window.alert("The LocalServer has no editable properties!");
        } else {
            window.openDialog
                ("chrome://annozilla/content/prefs/pref-annozilla-servers-edit.xul",
                 "serverEdit",
                 "chrome,modal=yes,resizable=no", uri);
        }
    }
}

/**
 * Removes a server entry from the servers
 * RDF file.
 *
 * @param aResourceString String name of resource to remove
 */
function deleteServerFromRDF(aResourceString) {

    // find and remove the entry
    var element = gRDF.GetResource(aResourceString);
    if (gSeq.IndexOf(element) != -1) {
        try {
            gSeq.RemoveElement(element, true);
        }
        catch(e) {
            dump("Error removing element: " + e + "\n");
        } 
    }
    else {
        dump("Error: Could not find element in container.\n");
    }

    // remove the resource from the graph  

    // urns:  [ URN predicate function for namespace, [ "literal0", "literal1", ... ] ]
    var urns = [ [SERVER_RDF, ["user", "password"]] ];

    // iterate over all namespaces
    for (var i = 0; i < urns.length; i++) {

        var namespaceFunc = urns[i][0];
        var properties = urns[i][1];

        // iterate over all properties in the namespace
        for (var j = 0; j < properties.length; j++) {
            var propertyRes = gRDF.GetResource(namespaceFunc(properties[j]), true);
            var value = gDS.GetTarget(element, propertyRes, true);
            if (element && propertyRes && value) {
                gDS.Unassert(element, propertyRes, value, true);
            }
        }
    }

    // flush data source
    try {
        gDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
    } catch(e) {
        dump("Couldn't flush data-source.\n");
    }
}


/**
 * Creates a new blank servers file.
 *
 * @param file nsIFile with location for new file
 * @return the nsIFile (evals to a nsILocalFile) created
 */
function createNewAnnozillaServersFile(file) {

    try {

        // initialize local file
        var localFile = Components.classes[NS_LOCAL_FILE_CONTRACTID].createInstance(Components.interfaces.nsILocalFile);

        localFile.initWithPath(file.path);

        if (localFile.exists() && 
                (localFile.isDirectory() || !localFile.isWritable())) {
            throw Components.results.NS_ERROR_FAILURE;        
        }

        // write flags
        var flags = NS_WRONLY | NS_CREATE_FILE | NS_TRUNCATE;

        // get a file output stream
        var fileStream =
            Components.classes[NS_LOCALFILEOUTPUTSTREAM_CONTRACTID].
                createInstance(Components.interfaces.nsIFileOutputStream);
        fileStream.init(localFile, flags, DEFAULT_FILE_PERMS, 0);

        // template for servers rdf
        var outputStr = "<?xml version=\"1.0\"?>\n" +
            "<RDF:RDF xmlns:s=\"urn:annotations:serverNS#\"  \n" +
            "   xmlns:NC=\"http://home.netscape.com/NC-rdf#\"  \n" +
            "   xmlns:RDF=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\n" +
            "   <RDF:Bag about=\"urn:annotations:servers\">\n" +
            "   </RDF:Bag>\n" +
            "</RDF:RDF>\n";

        fileStream.write(outputStr, outputStr.length);
        fileStream.close();

        return localFile;
    }
    catch (e) {
        dump("Error writing out new file: " + e + "\n");
        return undefined;
    }
}

/**
 * Take the selected server value from the list box,
 * and set the Default Post Server to its URI.
 */
function setDefaultPostServer () {
    // Set the textbox
    document.getElementById ("defaultPostServer").value =
        document.getElementById("serverList").selectedItems[0].label;
    // On FF 1.5+, also have to inform the pane of the change
    if (document.getElementById ("annozillaServerPreferencesPane")) {
        document.getElementById ("annozillaServerPreferencesPane").userChangedValue
            (document.getElementById ("defaultPostServer"));
    }
}

/**
 * Set the "disabled" attribute of the broadcaster
 * depending on whether any items are selected or not.
 * (Used to make 'Edit' and 'Remove' disabled when
 * no items are selected.)
 */
function serverSelect (listbox) {
    document.getElementById("serverSelected").setAttribute
        ("disabled",
         listbox.selectedItems.length == 0);
}

/**
 * Called whenever any of the items in the "Basic Server Prefs"
 * fields are changed. Note that the preferences are written
 * immediately (assuming the passwords match), not on OK/Apply.
 */
function setBasicServer () {

    if (document.getElementById("password").value ==
        document.getElementById("confirmation").value) {

        document.getElementById("nomatchWarning").
           setAttribute ("style", "display: none");

    } else {

        document.getElementById("nomatchWarning").
           setAttribute ("style", "display: block; font-weight: bold; color: red; font-size: larger;");
       return;
    }

    var elements = gSeq.GetElements();
    while (elements.hasMoreElements())
    {
        var server = elements.getNext();
        var serverURL = server.QueryInterface(Components.interfaces.nsIRDFResource).Value;

        // Whether the server URL has changed or not, it's still easier to
        // delete the existing server and add a new one
        deleteServerFromRDF(serverURL);
    }

    var serverType = document.getElementById("basicServerType").value;

    var newServer, user, password;
    if (serverType == "local")
    {
        newServer = "urn:localserver";
        user = "noUser";
        password = "noPass";
    }
    else
    {
        newServer = document.getElementById("serverURL").value;
        user = document.getElementById("user").value;
        password =  document.getElementById("password").value;
    }

    // get the Bag in container
    var container = Components.classes["@mozilla.org/rdf/container;1"].createInstance();
    container = container.QueryInterface(Components.interfaces.nsIRDFContainer);

    // initialize the container to the root urn
    var containerRes = gRDF.GetResource(SERVERS_ROOT_URN);
    container.Init(gDS, containerRes);

    var newServerResource = gRDF.GetResource (newServer);

    container.AppendElement(newServerResource);

    assertForResource(newServerResource, "user", user);
    assertForResource(newServerResource, "password", password);

    var remoteDS = gDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    remoteDS.Flush();

    // Everything else has been written directly.
    // The default post server is set indirectly
    document.getElementById ("defaultPostServer").value = newServer;
    // On FF 1.5+, also have to inform the pane of the change
    if (document.getElementById ("annozillaServerPreferencesPane")) {
        document.getElementById ("annozillaServerPreferencesPane").userChangedValue
            (document.getElementById ("defaultPostServer"));
    }

}
